//--------------------------
//- spi.c                  -
//-                        -
//-	Created on: 08.12.2011 -
//-	    Author: Alex Tofan -
//--------------------------

#include "spi.h"
#include "xil_io.h"
#include "xparameters.h"
#include "stdio.h"
#include "stdlib.h"

//-----------------------------------------
//--- SPI Initialization ------------------
//-----------------------------------------
int SPI_Init(void)
{
    int CfgValue = 0;
    // Configuration Register Settings
    CfgValue |= (0 << LSBFirst)         | // MSB First transfer format
                (1 << MasterTranInh)    | // Master transactions disabled
                (1 << ManualSlaveAssEn) | // Slave select output follows data in slave select register
                (0 << RxFifoReset)      | // Receive FIFO normal operation
                (0 << TxFifoReset)      | // Transmit FIFO normal operation
                (0 << CHPA)             | // Data valid on first SCK edge
                (0 << CPOL)             | // Active high clock, SCK idles low
                (1 << Master)           | // SPI in Master configuration mode
                (0 << SPE)              | // SPI enabled
                (0 << LOOP);              // Normal operation
    // Set corresponding value to the Configuration Register
    Xil_Out32(SPICR, CfgValue);
    return(1);
}

void SPI_SS0n_ON(void)
{
	// Enable slave (SSn = 0)
    Xil_Out32(SPISSR, 0x2);
}

void SPI_SS1n_ON(void)
{
	// Enable slave (SSn = 0)
    Xil_Out32(SPISSR, 0x1);
}

void SPI_SSn_OFF(void)
{
	// Disable slave (SSn = 1)
    Xil_Out32(SPISSR, 0x3);
}

int SPI_TransferData(char ss)
{
    int CfgValue  = 0;
    int SPIStatus = 0;
    int SPIRead   = 0;
    int SPIData   = 0;
    // Write initial data to master SPIDTR register
    Xil_Out32(SPIDTR, SPIData);
    // Ensure the SPISSR register has all ones
    SPI_SSn_OFF();
    // Write configuration data to master SPI device SPICR
    CfgValue |= (0 << LSBFirst)         | // MSB First transfer format
                (1 << MasterTranInh)    | // Master transactions disabled
                (1 << ManualSlaveAssEn) | // Slave select output follows data in slave select register
                (0 << RxFifoReset)      | // Receive FIFO normal operation
                (0 << TxFifoReset)      | // Transmit FIFO normal operation
                (0 << CHPA)             | // Data valid on first SCK edge
                (0 << CPOL)             | // Active high clock, SCK idles low
                (1 << Master)           | // SPI in Master configuration mode
                (1 << SPE)              | // SPI enabled
                (0 << LOOP);              // Normal operation
    Xil_Out32(SPICR, CfgValue);
    // Write to SPISSR to manually assert SSn
    switch(ss)
    {
    	case 0:
    		SPI_SS0n_ON();
    		break;
    	case 1:
    		SPI_SS1n_ON();
    		break;
    	default:
    		break;
    }
    // Write same configuration in SPICR but clear inhibit bit
    CfgValue &= ~(1 << MasterTranInh);
    Xil_Out32(SPICR, CfgValue);
    // Poll status for completion
    do
    {
        SPIStatus = Xil_In32(SPISR);
    }
    while((SPIStatus & 0x2) != 2);
    // Write all ones to SPISSR
    SPI_SSn_OFF();
    // Read received data from SPI Core buffer
    SPIRead = Xil_In32(SPIDRR);
    return(SPIRead);
}

void delay_ms(u32 ms_count)
{
  u32 count;
  for (count = 0; count < ((ms_count * 100000) + 1); count++) {
    asm("nop");
  }
}

int GetVin1(void)
{
	int RxData;
	RxData = (((SPI_TransferData(0) & 0x00000FF0) >> 4) << 24) |
			 (((SPI_TransferData(0) & 0x00000FF0) >> 4) << 16) |
			 (((SPI_TransferData(0) & 0x00000FF0) >> 4) << 8)  |
			 ((SPI_TransferData(0) & 0x00000FF0) >> 4);
	return(RxData);
}
void PrintVin1(void)
{
	int x;
	char c[] = {'0','.','0','0','0','0',0};
	int nr;
	int i = 5;
	x = SPI_TransferData(0);
	nr = x * 33000 / 4096;
	while(nr>0)
	{
		c[i] = (nr % 10) + 48;
		nr = nr / 10;
		i = i - 1;
		// Skip the 2nd position (it is pre-loaded with '.')
		if(i == 1)
		{
			i = i-1;
		}
	}
	xil_printf("Vin1 = %s V\n\r",c);
}
void PrintVin2(void)
{
	int x;
	char c[] = {'0','.','0','0','0','0',0};
	int nr;
	int i = 5;
	x = SPI_TransferData(1);
	nr = x * 33000 / 4096;
	while(nr>0)
	{
		c[i] = (nr % 10) + 48;
		nr = nr / 10;
		i = i - 1;
		// Skip the 2nd position (it is pre-loaded with '.')
		if(i == 1)
		{
			i = i-1;
		}
	}
	xil_printf("Vin2 = %s V\n\r",c);
}
